==> crashops_flutter-0.0.825.tar.gz: lib/crashops_flutter.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:stack_trace/stack_trace.dart';

typedef COErrorCallback = Function(FlutterErrorDetails flutterErrorDetails,
    dynamic globalError, StackTrace stackTrace);

/// The plugin's façade - Use this class to configure CrashOps SDK.

class CrashOps {
  static const STATUS_OK = "OK";

  static get instance => CrashOps();
  static get shared => CrashOps();

  /// Returns the instance of CrashOps plugin
  factory CrashOps() {
    return _singleton;
  }

  static const MethodChannel _channel = const MethodChannel('crashops_flutter');

  /// Determines whether CrashOps will / won't collect crashes / errors on debug mode. The default value is `false`.
  bool isEnabledInDebugMode = false;

  /// Determines whether CrashOps will / won't operate, useful if you wish to remotely set it off. The default value is `true`.
  Future<bool> get isEnabled async {
    if (_isEnabled != null) return _isEnabled;

    _isEnabled = await _channel.invokeMethod('CrashOps#isEnabled');

    return _isEnabled;
  }

  /// Sets the CrashOps behaviour, useful if you wish to remotely set it off. The default value is `true`.
  Future<bool> setEnabled(bool value) async {
    var result = await _channel.invokeMethod('CrashOps#setEnabled', value);
    bool didUpdate = result == STATUS_OK;
    if (didUpdate) {
      _isEnabled = value;
    }

    return didUpdate;
  }

  /// Appends new metadata to existing metadata.
  /// This lets you attach any data if you like (optional).
  ///
  /// This extra data will be included in all future reports.
  Future<bool> setMetadata(Map<String, dynamic> metadata) async {
    var result = await _channel.invokeMethod('CrashOps#setMetadata', metadata);
    return result == STATUS_OK;
  }

  /// Sets the client ID (recommended). Mandatory only if you wish to store your logs in CrashOps servers.
  Future<bool> setClientId(String clientId) async {
    var result = await _channel.invokeMethod('CrashOps#setClientId', clientId);
    return result == STATUS_OK;
  }

  Future<bool> logError(
      {String errorMessage, Map<String, dynamic> extraInfo}) async {
    if (errorMessage == null) return false;
    StackTrace stackTrace = StackTrace.current;

    return await _logError(
        errorMessage: errorMessage,
        extraInfo: extraInfo,
        context: "none",
        stackTrace: stackTrace);
  }

  Future<bool> _logError(
      {String errorMessage,
      Map<String, dynamic> extraInfo,
      dynamic context,
      StackTrace stackTrace}) async {
    if (errorMessage == null) return false;
    String result;

    final List<String> stackTraceLines =
        Trace.format(stackTrace).trimRight().split('\n');

    // Yes, we learned from the best:
    // https://pub.dev/documentation/firebase_crashlytics/latest/firebase_crashlytics/Crashlytics/getStackTraceElements.html
    final List<Map<String, String>> stackTraceElements =
        _getStackTraceElements(stackTraceLines);

    extraInfo ??= {};
    result = await _channel
        .invokeMethod('CrashOps#onFlutterError', <String, dynamic>{
      'exception': errorMessage,
      'extraInfo': extraInfo,
      'context': context,
      'stackTraceElements': stackTraceElements,
    });

    return result == STATUS_OK;
  }

  /// Programmatically logs a non-fatal error, this error also sent to CrashOps servers in case that `clientId` exists.
  /// Discussion: The error could be anything, as the `exception` field appears in FlutterErrorDetails (https://api.flutter.dev/flutter/foundation/FlutterErrorDetails/exception.html).
  Future<bool> _onError(dynamic error, [StackTrace stackTrace]) async {
    if (error == null) return false;

    stackTrace ??= StackTrace.current;
    String result;

    FlutterErrorDetails errorDetails;
    if (error is FlutterErrorDetails) {
      errorDetails = error;
    } else {
      errorDetails = FlutterErrorDetails(exception: error, stack: stackTrace);
    }

    if (inDebugMode && !isEnabledInDebugMode) {
      print(errorDetails);
      result = STATUS_OK;
    } else {
      bool isLogged = await _logError(
          errorMessage: errorDetails.exceptionAsString() ?? "none",
          extraInfo: {"exception": errorDetails.exception},
          context: '${errorDetails.context ?? "none"}',
          stackTrace: errorDetails.stack);
      if (isLogged) {
        result = STATUS_OK;
      }
    }

    return result == STATUS_OK;
  }

  /*  Private methods  */

  bool _isEnabled;
  bool get inDebugMode => _inDebugMode;
  bool _inDebugMode;

  static final CrashOps _singleton = new CrashOps._internal();

  CrashOps._internal() {
    _isEnabled = null;
    _inDebugMode = false;
    assert(_inDebugMode = true);
  }

  static List<Map<String, String>> _getStackTraceElements(List<String> lines) {
    final List<Map<String, String>> elements = <Map<String, String>>[];
    for (String line in lines) {
      final List<String> lineParts = line.split(RegExp('\\s+'));
      try {
        final String fileName = lineParts[0];
        final String lineNumber = lineParts[1].contains(":")
            ? lineParts[1].substring(0, lineParts[1].indexOf(":")).trim()
            : lineParts[1];

        final Map<String, String> element = <String, String>{
          'file': fileName,
          'line': lineNumber,
        };

        if (lineParts[2].contains(".")) {
          final String className =
              lineParts[2].substring(0, lineParts[2].indexOf(".")).trim();
          final String methodName =
              lineParts[2].substring(lineParts[2].indexOf(".") + 1).trim();

          element['class'] = className;
          element['method'] = methodName;
        } else {
          element['method'] = lineParts[2];
        }

        elements.add(element);
      } catch (e) {
        print(e.toString());
      }
    }
    return elements;
  }

  void run(
      {Widget app, bool withCatchingErrors = true, COErrorCallback onError}) {
    if (!withCatchingErrors) {
      runApp(app);
      if (onError != null) {
        // We should let the developer know that this `onError` closure is actually redundant because CrashOps won't intercept errors anyway.
        onError(
            null,
            "CrashOps configured without catching errors but using `onError` closure. Besides this call CrashOps will ignore all errors.",
            StackTrace.current);
      }
    } else {
      var previousOnErrorCallback = FlutterError.onError;

      // Catches only Flutter errors
      FlutterError.onError = (FlutterErrorDetails details) {
        _onError(details);
        if (onError != null) {
          onError(details, details.exception, details.stack);
        }
        FlutterError.dumpErrorToConsole(details, forceReport: false);
        if (previousOnErrorCallback != null &&
            FlutterError.dumpErrorToConsole != previousOnErrorCallback) {
          previousOnErrorCallback(details);
        }
      };

      // Catches all Dart
      runZoned<Future<void>>(() async {
        runApp(app);
      }, onError: (error, stackTrace) {
        // This catches also Dart errors, not only Flutter errors.
        // For more details, read: https://flutter.dev/docs/cookbook/maintenance/error-reporting
        _onError(error, stackTrace);

        FlutterErrorDetails details;
        if (error is FlutterErrorDetails) {
          details = error;

          FlutterError.dumpErrorToConsole(details, forceReport: false);
          if (previousOnErrorCallback != null &&
              FlutterError.dumpErrorToConsole != previousOnErrorCallback) {
            previousOnErrorCallback(details);
          }
        }

        if (onError != null) {
          onError(details, error, stackTrace);
        }
      });
    }
  }
}

==> crashops_flutter-0.0.826.tar.gz: lib/crashops_flutter.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/services.dart';
import 'package:flutter/widgets.dart';
import 'package:stack_trace/stack_trace.dart';

typedef COErrorCallback = Function(FlutterErrorDetails flutterErrorDetails,
    dynamic globalError, StackTrace stackTrace);

/// The plugin's façade - Use this class to configure CrashOps SDK.

class CrashOps {
  static const STATUS_OK = "OK";

  static get instance => CrashOps();
  static get shared => CrashOps();

  /// Returns the instance of CrashOps plugin
  factory CrashOps() {
    return _singleton;
  }

  static const MethodChannel _channel = const MethodChannel('crashops_flutter');

  /// Determines whether CrashOps will / won't collect crashes / errors on debug mode. The default value is `false`.
  bool isEnabledInDebugMode = false;

  /// Determines whether CrashOps will / won't operate, useful if you wish to remotely set it off. The default value is `true`.
  Future<bool> get isEnabled async {
    if (_isEnabled != null) return _isEnabled;

    _isEnabled = await _channel.invokeMethod('CrashOps#isEnabled');

    return _isEnabled;
  }

  /// Sets the CrashOps behaviour, useful if you wish to remotely set it off. The default value is `true`.
  Future<bool> setEnabled(bool value) async {
    var result = await _channel.invokeMethod('CrashOps#setEnabled', value);
    bool didUpdate = result == STATUS_OK;
    if (didUpdate) {
      _isEnabled = value;
    }

    return didUpdate;
  }

  /// Appends new metadata to existing metadata.
  /// This lets you attach any data if you like (optional).
  ///
  /// This extra data will be included in all future reports.
  Future<bool> setMetadata(Map<String, dynamic> metadata) async {
    var result = await _channel.invokeMethod('CrashOps#setMetadata', metadata);
    return result == STATUS_OK;
  }

  /// Sets the application key. Mandatory only if you wish to store your logs in CrashOps servers.
  Future<bool> setApplicationKey(String appKey) async {
    var result = await _channel.invokeMethod('CrashOps#setAppKey', appKey);
    return result == STATUS_OK;
  }

  Future<bool> logError(
      {String errorMessage, Map<String, dynamic> extraInfo}) async {
    if (errorMessage == null) return false;
    StackTrace stackTrace = StackTrace.current;

    return await _logError(
        errorMessage: errorMessage,
        extraInfo: extraInfo,
        context: "none",
        stackTrace: stackTrace);
  }

  Future<bool> _logError(
      {String errorMessage,
      Map<String, dynamic> extraInfo,
      dynamic context,
      StackTrace stackTrace}) async {
    if (errorMessage == null) return false;
    String result;

    final List<String> stackTraceLines =
        Trace.format(stackTrace).trimRight().split('\n');

    // Yes, we learned from the best:
    // https://pub.dev/documentation/firebase_crashlytics/latest/firebase_crashlytics/Crashlytics/getStackTraceElements.html
    final List<Map<String, String>> stackTraceElements =
        _getStackTraceElements(stackTraceLines);

    extraInfo ??= {};
    result = await _channel
        .invokeMethod('CrashOps#onFlutterError', <String, dynamic>{
      'exception': errorMessage,
      'extraInfo': extraInfo,
      'context': context,
      'stackTraceElements': stackTraceElements,
    });

    return result == STATUS_OK;
  }

  /// Programmatically logs a non-fatal error, this error also sent to CrashOps servers in case that `clientId` exists.
  /// Discussion: The error could be anything, as the `exception` field appears in FlutterErrorDetails (https://api.flutter.dev/flutter/foundation/FlutterErrorDetails/exception.html).
  Future<bool> _onError(dynamic error, [StackTrace stackTrace]) async {
    if (error == null) return false;

    stackTrace ??= StackTrace.current;
    String result;

    FlutterErrorDetails errorDetails;
    if (error is FlutterErrorDetails) {
      errorDetails = error;
    } else {
      errorDetails = FlutterErrorDetails(exception: error, stack: stackTrace);
    }

    if (inDebugMode && !isEnabledInDebugMode) {
      print(errorDetails);
      result = STATUS_OK;
    } else {
      bool isLogged = await _logError(
          errorMessage: errorDetails.exceptionAsString() ?? "none",
          extraInfo: {"exception": errorDetails.exception},
          context: '${errorDetails.context ?? "none"}',
          stackTrace: errorDetails.stack);
      if (isLogged) {
        result = STATUS_OK;
      }
    }

    return result == STATUS_OK;
  }

  /*  Private methods  */

  bool _isEnabled;
  bool get inDebugMode => _inDebugMode;
  bool _inDebugMode;

  static final CrashOps _singleton = new CrashOps._internal();

  CrashOps._internal() {
    _isEnabled = null;
    _inDebugMode = false;
    assert(_inDebugMode = true);
  }

  static List<Map<String, String>> _getStackTraceElements(List<String> lines) {
    final List<Map<String, String>> elements = <Map<String, String>>[];
    for (String line in lines) {
      final List<String> lineParts = line.split(RegExp('\\s+'));
      try {
        final String fileName = lineParts[0];
        final String lineNumber = lineParts[1].contains(":")
            ? lineParts[1].substring(0, lineParts[1].indexOf(":")).trim()
            : lineParts[1];

        final Map<String, String> element = <String, String>{
          'file': fileName,
          'line': lineNumber,
        };

        if (lineParts[2].contains(".")) {
          final String className =
              lineParts[2].substring(0, lineParts[2].indexOf(".")).trim();
          final String methodName =
              lineParts[2].substring(lineParts[2].indexOf(".") + 1).trim();

          element['class'] = className;
          element['method'] = methodName;
        } else {
          element['method'] = lineParts[2];
        }

        elements.add(element);
      } catch (e) {
        print(e.toString());
      }
    }
    return elements;
  }

  void run(
      {Widget app, bool withCatchingErrors = true, COErrorCallback onError}) {
    if (!withCatchingErrors) {
      runApp(app);
      if (onError != null) {
        // We should let the developer know that this `onError` closure is actually redundant because CrashOps won't intercept errors anyway.
        onError(
            null,
            "CrashOps configured without catching errors but using `onError` closure. Besides this call CrashOps will ignore all errors.",
            StackTrace.current);
      }
    } else {
      var previousOnErrorCallback = FlutterError.onError;

      // Catches only Flutter errors
      FlutterError.onError = (FlutterErrorDetails details) {
        _onError(details);
        if (onError != null) {
          onError(details, details.exception, details.stack);
        }
        FlutterError.dumpErrorToConsole(details, forceReport: false);
        if (previousOnErrorCallback != null &&
            FlutterError.dumpErrorToConsole != previousOnErrorCallback) {
          previousOnErrorCallback(details);
        }
      };

      // Catches all Dart
      runZoned<Future<void>>(() async {
        runApp(app);
      }, onError: (error, stackTrace) {
        // This catches also Dart errors, not only Flutter errors.
        // For more details, read: https://flutter.dev/docs/cookbook/maintenance/error-reporting
        _onError(error, stackTrace);

        FlutterErrorDetails details;
        if (error is FlutterErrorDetails) {
          details = error;

          FlutterError.dumpErrorToConsole(details, forceReport: false);
          if (previousOnErrorCallback != null &&
              FlutterError.dumpErrorToConsole != previousOnErrorCallback) {
            previousOnErrorCallback(details);
          }
        }

        if (onError != null) {
          onError(details, error, stackTrace);
        }
      });
    }
  }
}
